/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   *
 * http://www.gnu.org/software/gnugo/ for more information.      *
 *                                                               *
 * Copyright 1999 by the Free Software Foundation.               *
 *                                                               *
 * This program is free software; you can redistribute it and/or *
 * modify it under the terms of the GNU General Public License   *
 * as published by the Free Software Foundation - version 2.     *
 *                                                               *
 * This program is distributed in the hope that it will be       *
 * useful, but WITHOUT ANY WARRANTY; without even the implied    *
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR       *
 * PURPOSE.  See the GNU General Public License in file COPYING  *
 * for more details.                                             *
 *                                                               *
 * You should have received a copy of the GNU General Public     *
 * License along with this program; if not, write to the Free    *
 * Software Foundation, Inc., 59 Temple Place - Suite 330,       *
 * Boston, MA 02111, USA                                         *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include <assert.h>
#include <malloc.h>
#include "liberty.h"

/* This is the maximum number of liberties that we care to keep track of. */
#define MAX_LIBERTIES 20

/* Size of string table. */
#define MAX_STRINGS (2 * MAX_BOARD * MAX_BOARD / 3)


/* Incremental string data. */
struct string_data {
  int color;                       /* Color of string, BLACK or WHITE */
  int size;                        /* Number of stones in string. */
  int origini;                     /* Coordinates of "origin", i.e. */
  int originj;                     /* "upper left" stone. */
  int liberties;                   /* Number of liberties. */
  int libi[MAX_LIBERTIES];         /* Coordinates of liberties. */
  int libj[MAX_LIBERTIES];
  int neighbors;                   /* Number of neighbor strings */
  int neighborlist[MAXCHAIN];      /* List of neighbor string numbers. */
  int mark;                        /* General purpose mark. */
};


/* we keep the adress and the old value */
struct change_stack_entry {
  int *address;
  int value;
};

/* we keep the adress and the old value */
struct vertex_stack_entry {
  board_t *address;
  int value;
};

#define STACK_SIZE 10000

extern struct change_stack_entry change_stack[STACK_SIZE];
extern struct vertex_stack_entry vertex_stack[STACK_SIZE];

extern int change_stack_index;
extern int vertex_stack_index;

#define CLEAR_STACKS()\
(change_stack_index = 0, vertex_stack_index = 0)

/* Begin a record : adress==NULL */
#define BEGIN_CHANGE_RECORD()\
(change_stack[change_stack_index++].address = NULL,\
 vertex_stack[vertex_stack_index++].address = NULL)

/* Save a value : store the adress and the value in the stack */
#define PUSH_VALUE(v)\
(change_stack[change_stack_index].address = &(v),\
 change_stack[change_stack_index++].value = (v))

/* Save a board value : store the adress and the value in the stack */
#define PUSH_VERTEX(v)\
(vertex_stack[vertex_stack_index].address = &(v),\
vertex_stack[vertex_stack_index++].value = (v))

#define POP_MOVE()\
  while (change_stack[--change_stack_index].address)\
  *(change_stack[change_stack_index].address) =\
  change_stack[change_stack_index].value

#define POP_VERTICES()\
  while (vertex_stack[--vertex_stack_index].address)\
  *(vertex_stack[vertex_stack_index].address) =\
  vertex_stack[vertex_stack_index].value


/*
 * Local Variables:
 * tab-width: 8
 * c-basic-offset: 2
 * End:
 */
